# -*- coding: utf-8 -*-

'''
Created on 13 may 2013

@author: karli.sjoberg@slu.se

version: 1.0
date: 31 may 2013
comment: Here is the K(C)ustomer Interactive Resource Aide. It´s not pretty,
         or elegant. Can hardly be called "Pythonic", whatever that means.
         But it does what it´s supposed to do, and I´m happy with that, for
         now...
'''

import time
import smtplib
import logging
logging.basicConfig(filename='/var/log/vpscreate.log', format='%(asctime)s: %(message)s', level=logging.DEBUG)
from ConfigParser import SafeConfigParser
parser = SafeConfigParser()
parser.read("/etc/vpscreate/vpscreate.conf")

from ovirtsdk.api import API
from ovirtsdk.xml import params

URL             = parser.get('vpscreate', 'url')
CACERT          = parser.get('vpscreate', 'cacert')
USERNAME        = parser.get('vpscreate', 'username')
PASSWORD        = parser.get('vpscreate', 'password')
CLUSTER_NAME    = parser.get('vpscreate', 'cluster_name')
SNAPSHOT_NAME   = parser.get('vpscreate', 'snapshot_name')
ROLE_NAME       = parser.get('vpscreate', 'role_name')
DOMAIN_NAME     = parser.get('vpscreate', 'domain_name')

api = API(url=URL, ca_file=CACERT, username=USERNAME, password=PASSWORD)

class CreateClass(object):
    
    def PrintError(self, VM_NAME, CUSTOMER_ID, ERRORCODE):
        print "Content-type:text/html\r\n\r\n"
        print """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-type" content="text/html; charset=utf-8" />
<title>Order Portal</title>
<link rel="stylesheet" href="../../style.css" type="text/css" media="screen, projection" charset="utf-8" />
</head>
<body>
<div>
<div class="left">
</div>
<div class="center">
</div>
</div>
<div class="main">
<div class="welcome">VPS Order Portal</div>"""
        print {
               '0': '<p>Template name was empty. You must go back and choose which type of VPS you want. Sorry :(</p>',
               '1': '<p>User name was empty. You must go back and type in your user name in the correct field. Sorry :(</p>',
               '2': '<p>VM name was empty. You must go back and type in the name of VPS in the correct field. Sorry :(</p>',
               '3': '<p>Your desired VM alias "%s.foo.bar" is already registered in DNS. You must go back and choose a different name for your VPS. Sorry :(</p>' % (VM_NAME),
               '4': '<p>Your desired VM name "%s" is already registered in oVirt. You must go back and choose a different name for your VPS. Sorry :(</p>' % (VM_NAME),
               }[ERRORCODE]
        print """<b><a href="http://foo.bar/orderportal.html">Go Back</a></b>
</div>
</body>
</html>"""

    def CreateVM(self,VM_NAME,CLUSTER_NAME,TEMPLATE_NAME):
        try:
            api.vms.add(params.VM(name=VM_NAME,cluster=api.clusters.get(CLUSTER_NAME),template=api.templates.get(TEMPLATE_NAME)))
            logging.debug('"%s" was created from Template successfully' % (VM_NAME))
            logging.debug('Waiting for VM to reach Down status')
            while api.vms.get(VM_NAME).status.state != 'down':
                time.sleep(1)
        except Exception as e:
            logging.debug('Failed to create VM from Template:\n%s' % str(e))
            exit(1)
            
    def CreateSnapshot(self, VM_NAME, SNAPSHOT_NAME):
        try:
            api.vms.get(VM_NAME).snapshots.add(params.Snapshot(description=SNAPSHOT_NAME, vm=api.vms.get(VM_NAME)))
            logging.debug('Creating a Snapshot called %s' % (SNAPSHOT_NAME))
            logging.debug('Waiting for Snapshot creation to finish')
            while api.vms.get(VM_NAME).snapshots.list()[1].get_snapshot_status() == 'locked':
                time.sleep(1)
            logging.debug('Snapshot creation complete')
        except Exception as e:
            logging.debug('Failed to Create a Snapshot:\n%s' % str(e))
            exit(1)
            
    def CreateUser(self, CUSTOMER_ID, DOMAIN_NAME):
        try:
            logging.debug('Creating a user called %s' % (CUSTOMER_ID))
            domain = api.domains.get(name='%s' % (DOMAIN_NAME))
            userparams = params.User()
            userparams.set_user_name(CUSTOMER_ID)
            userparams.set_domain(domain)
            api.users.add(userparams)
        except Exception as e:
            logging.debug('Failed to create a user:\n%s' % str(e))
            exit(1)
            
    def SetPermissions(self, VM_NAME, CUSTOMER_ID, ROLE_NAME):
        try:
            logging.debug('Setting Owner permissions on VM %s for user %s' % (VM_NAME,CUSTOMER_ID))
            vm = api.vms.get(name=VM_NAME)
            user = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0]
            role = api.roles.get(name='%s' % (ROLE_NAME))
            permit = params.Permission()
            permit.set_role(role)
            permit.set_vm(vm)
            user.permissions.add(permit)
        except Exception as e:
            logging.debug('Failed to set permissions:\n%s' % str(e))
            exit(1)
            
    def StartVM(self, VM_NAME):
        try:
            if api.vms.get(VM_NAME).status.state != 'up':
                logging.debug('Starting VM')
                api.vms.get(VM_NAME).start()
                logging.debug('Waiting for VM to reach Up status')
                while api.vms.get(VM_NAME).status.state != 'up':
                    time.sleep(1)
            else:
                logging.debug('VM already up')
        except Exception as e:
            logging.debug('Failed to Start VM:\n%s' % str(e))
            exit(1)

    def GetHostname(self, VM_NAME):
        try:
            logging.debug('Getting hostname for VM "%s"' % (VM_NAME))
            vm = api.vms.get(VM_NAME)
            macadress = vm.nics.get(name='*').get_mac().get_address()
            splitstring = macadress.split(":")
            concatstring = [ i.lstrip('0') for i in splitstring ]
            separator = "-"
            joinedstring = separator.join(concatstring)
            return "ovirt-0%s.foo.bar" % (joinedstring)
        except Exception as e:
            logging.debug('Failed to get hostname:\n%s'  % str(e))
            exit(1)
            
    def SetOwner(self, CUSTOMER_ID, HOSTNAME, VM_NAME):
        try:
            logging.debug('Setting user "%s" as owner on VM "%s"' % (CUSTOMER_ID,VM_NAME))
            fo = open("/home/owners/owners.txt", "a")
            fo.write("%s\t\t%s\t%s.foo.bar\n" % (CUSTOMER_ID,HOSTNAME,VM_NAME))
            fo.close()
        except Exception as e:
            logging.debug('Failed to set owner:\n%s'  % str(e))
            exit(1)

    def SendAliasRegistration(self, CUSTOMER_ID, VM_NAME, HOSTNAME): 
        try:
            logging.debug('Sending registration mail to domainmaster')
            smtpObj = smtplib.SMTP('smtp.foo.bar', 25)
            sender = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_email()
            sender_firstname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_name().encode('utf-8')
            sender_lastname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_last_name().encode('utf-8')
            receiver = 'domainmaster@foo.bar'
            message = """From: %s %s <%s>
To: %s
Subject: VPS alias registration

%s.foo.bar. CNAME %s.""" % (sender_firstname,sender_lastname,sender,receiver,VM_NAME,HOSTNAME)
            smtpObj.sendmail(sender, receiver, message)
            logging.debug('Successfully sent alias registration email')
        except Exception as e:
            logging.debug('Failed to send alias registration email:\n%s' % str(e))
 
    def SendConfirmation(self, CUSTOMER_ID, VM_NAME, HOSTNAME):
        smtpObj = smtplib.SMTP('smtp.foo.bar', 25)
        sender = 'vpsorderportal@foo.bar'
        receiver = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_email()
        receiver_firstname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_name().encode('utf-8')
        receiver_lastname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_last_name().encode('utf-8')
        message = """From: VPS Orderportal <%s>
To: %s %s <%s>
Subject: Your VPS "%s" is ready!

Thank you %s!

Your VPS´s alias name "%s.foo.bar" has been sent to domainmaster@ for registration. In the mean time, you can access the machine through the Userportal, or with it´s real domain adress, "%s".

To start using your new VPS, you must first go to "https://foo.bar/spice.html" and install a plugin for your web browser that allows you to connect to machine´s console(the monitor). This only needs to be done once. You will then need to shut down all your open brower sessions and start it back up again for the plugin to be loaded. After that, you go to "https://foo.bar/UserPortal" where you log in using your domain account to administer the machine. You can also log in to the machine over SSH(if *NIX) or PowerShell(if Windows) from anywhere, not having to go through the portal. Regardless of whether you log in through the portal or remote, you just use your domain account and you´re in! To elevate your privileges, use sudo(if *NIX) or just "Run as Administrator"(if Windows).

To take a snapshot(restore point) of the machine, you have to temporarily shut it down, take the snapshot, and then power it back up again. At what time and how often snapshots are taken, you decide for yourself, it´s your machine, but we from the IT-Department thinks that it´s an important thing to do regurarly in case something goes wrong and you want to go back in time to correct the problem.

If you'd like to stop using your VPS and delete it permanently, just go back to the orderportal, scroll down to the bottom of the page and follow the instructions.

Common questions and tips are explained at "https://foo.bar/faq", and as usual you can send emails to our support "customerservice@foo.bar" or call "XXXXXXXXXXX".

Once again, thank you, and good luck!""" % (sender,receiver_firstname,receiver_lastname,receiver,VM_NAME,receiver_firstname,VM_NAME,HOSTNAME)
        try:
            smtpObj.sendmail(sender, receiver, message)         
            logging.debug('Successfully sent email')
        except Exception as e:
            logging.debug('Failed to send email:\n%s' % str(e))
            exit(1)

    def PrintSuccess(self):
        print "Content-type:text/html\r\n\r\n"
        print """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-type" content="text/html; charset=utf-8" />
<title>Order Portal</title>
<link rel="stylesheet" href="../../style.css" type="text/css" media="screen, projection" charset="utf-8" />
</head>
<body>
<div>
<div class="left">
</div>
<div class="center">
</div>
</div>
<div class="main">
<div class="welcome">VPS Order Portal</div>
<p>Your VPS has been created successfully! You will shortly receive an email explaining what you´ve gotten yourself into :)</p>
<p>You can close this page down now.</p>
</div>
</body>
</html>"""

class DestroyClass(object):
    
    def PrintError(self, VM_NAME, CUSTOMER_ID, ERRORCODE):
        print "Content-type:text/html\r\n\r\n"
        print """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-type" content="text/html; charset=utf-8" />
<title>Order Portal</title>
<link rel="stylesheet" href="../../style.css" type="text/css" media="screen, projection" charset="utf-8" />
</head>
<body>
<div>
<div class="left">
</div>
<div class="center">
</div>
</div>
<div class="main">
<div class="welcome">VPS Order Portal</div>"""
        print {
               '1': '<p>User name was empty. You must go back and type in your user name in the correct field. Sorry :(</p>',
               '2': '<p>VM name was empty. You must go back and type in the name of VPS in the correct field. Sorry :(</p>',
               '3': '<p>Your VM name "%s" is not registered in oVirt. You must go back and type in the correct name of your VPS. Sorry :(</p>' % (VM_NAME),
               '4': '<p>That user name "%s" is not registered in oVirt. You must go back and type in the correct owner user name of your VPS. Sorry :(</p>' % (CUSTOMER_ID),
               '5': '<p>Your user name is not owner of VM "%s". You must go back and type the correct owner of this VPS. Sorry :(</p>' % (VM_NAME),
               }[ERRORCODE]
        print """<b><a href="http://foo.bar/orderportal.html">Go Back</a></b>
</div>
</body>
</html>"""

    def GetHostname(self, VM_NAME):
        try:
            logging.debug('Getting hostname for VM "%s"' % (VM_NAME))
            vm = api.vms.get(VM_NAME)
            macadress = vm.nics.get(name='*').get_mac().get_address()
            splitstring = macadress.split(":")
            concatstring = [ i.lstrip('0') for i in splitstring ]
            separator = "-"
            joinedstring = separator.join(concatstring)
            return "ovirt-0%s.foo.bar" % (joinedstring)
        except Exception as e:
            logging.debug('Failed to get hostname:\n%s'  % str(e))
            exit(1)

    def deleteVM(self, VM_NAME):
        try:
            if api.vms.get(VM_NAME).status.state != 'down':
                api.vms.get(VM_NAME).stop()
                while api.vms.get(VM_NAME).status.state != 'down':
                    time.sleep(1)
            api.vms.get(VM_NAME).delete()
            logging.debug('"%s" was successfully deleted' % (VM_NAME))
        except Exception as e:
            logging.debug('Failed to delete VM:\n%s' % str(e))
            exit(1)

    def SendAliasDeregistration(self, CUSTOMER_ID, VM_NAME, HOSTNAME): 
        try:
            logging.debug('Sending registration mail to domainmaster')
            smtpObj = smtplib.SMTP('smtp.foo.bar', 25)
            sender = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_email()
            sender_firstname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_name().encode('utf-8')
            sender_lastname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_last_name().encode('utf-8')
            receiver = 'domainmaster@foo.bar'
            message = """From: %s %s <%s>
To: %s
Subject: VPS alias removal

%s.foo.bar. CNAME %s.""" % (sender_firstname,sender_lastname,sender,receiver,VM_NAME,HOSTNAME)
            smtpObj.sendmail(sender, receiver, message)
            logging.debug('Successfully sent alias de-registration email')
        except Exception as e:
            logging.debug('Failed to send alias de-registration email:\n%s' % str(e))

    def RemoveOwner(self, CUSTOMER_ID, HOSTNAME, VM_NAME):
        try:
            logging.debug('Removing user "%s" as owner on VM "%s"' % (CUSTOMER_ID,VM_NAME))
            fo = open("/home/owners/owners.txt")
            output = []
            for line in fo:
                if not HOSTNAME in line:
                    output.append(line)
            fo.close()
            fo = open("/home/owners/owners.txt", 'w')
            fo.writelines(output)
            fo.close()
        except Exception as e:
            logging.debug('Failed to remove user as owner:\n%s'  % str(e))
            exit(1)

    def SendDeleteConfirmation(self, CUSTOMER_ID, VM_NAME, HOSTNAME):
        smtpObj = smtplib.SMTP('smtp.foo.bar', 25)
        sender = 'vpsorderportal@foo.bar'
        receiver = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_email()
        receiver_firstname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_name().encode('utf-8')
        receiver_lastname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_last_name().encode('utf-8')
        message = """From: VPS Orderportal <%s>
To: %s %s <%s>
Subject: Your VPS "%s" has been marked for deletion!

Hi %s!

The VPS named "%s" has been marked for removal. If you have ordered this, please click on the link below. If not, disregard this message.

https://foo.bar/cgi-bin/vpsconfirmdestroy.cgi?userid=%s&vmname=%s""" % (sender,receiver_firstname,receiver_lastname,receiver,VM_NAME,receiver_firstname,VM_NAME,CUSTOMER_ID,VM_NAME)
        try:
            smtpObj.sendmail(sender, receiver, message)         
            logging.debug('Successfully sent email')
        except Exception as e:
            logging.debug('Failed to send email:\n%s' % str(e))
            exit(1)

    def SendDeleteSuccess(self, CUSTOMER_ID, VM_NAME, HOSTNAME):
        smtpObj = smtplib.SMTP('smtp.foo.bar', 25)
        sender = 'vpsorderportal@foo.bar'
        receiver = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_email()
        receiver_firstname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_name().encode('utf-8')
        receiver_lastname = api.users.list(query='usrname=%s*' % (CUSTOMER_ID))[0].get_last_name().encode('utf-8')
        message = """From: VPS Orderportal <%s>
To: %s %s <%s>
Subject: Your VPS "%s" has been deleted!

Hi %s!

Any VPS alias name "%s.foo.bar" has been sent to domainmaster@ for de-registration. You will receive confirmation as soon as the alias is removed and everything else has already been removed.

Thank you very much and we hope to see you again soon!""" % (sender,receiver_firstname,receiver_lastname,receiver,VM_NAME)
        try:
            smtpObj.sendmail(sender, receiver, message)         
            logging.debug('Successfully sent email')
        except Exception as e:
            logging.debug('Failed to send email:\n%s' % str(e))
            exit(1)

    def PrintDeleteSuccess(self, CODE):
        print "Content-type:text/html\r\n\r\n"
        print """<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
<meta http-equiv="Content-type" content="text/html; charset=utf-8" />
<title>Order Portal</title>
<link rel="stylesheet" href="../../style.css" type="text/css" media="screen, projection" charset="utf-8" />
</head>
<body>
<div>
<div class="left">
</div>
<div class="center">
</div>
</div>
<div class="main">
<div class="welcome">VPS Order Portal</div>"""
        print {
               '1': '<p>Your deletion request has been validated. You will shortly receive a confirmation email :)</p>',
               '2': '<p>Your VPS has been deleted successfully! Thank you! :)</p>',
               }[CODE]
        print """<p>You can close this page down now.</p>
</div>
</body>
</html>"""
